use strict;
use warnings;
use Test::More;

# generated by Dist::Zilla::Plugin::Test::PodSpelling 2.007006
use Test::Spelling 0.17;
use Pod::Wordlist;


add_stopwords(<DATA>);
all_pod_files_spelling_ok( qw( examples lib script t xt ) );
__DATA__
Adam
Alexandr
Bengen
Berry
BooK
Bowers
Bruhat
Bunce
Ciornii
Craig
David
Etheridge
Flavio
Golden
Graham
Hilko
Ingy
James
Karen
Keenan
Kennedy
Knop
Kogman
Mengué
Monfils
Müller
Nathan
Neil
Net
Olivier
Philippe
Poletti
Rezic
Richlv
Slaven
Steffen
Tim
Tiny
YAML
Yuval
adamk
bengen
book
chorny
craigberry
dagolden
dolmen
döt
ether
flavio
haarg
ingy
irc
jkeenan
lib
neil
nmo
nothingmuch
richlv
slaven
smueller
יובל
קוג
